from mext.xmlgen import (
    XmlElementGen, XmlEntity,
    XmlDocument, XmlElement,
    EscapedXML as raw_html,
    as_xml as as_xhtml,
    NOTAG
)
    #, xmlelem as h


_non_empty_html_tags = frozenset('''
  a abbr acronym address applet b bdo big blockquote body button
  caption center cite code colgroup dd dfn div dl dt em fieldset font
  form frameset h1 h2 h3 h4 h5 h6 head html i iframe ins kbd label
  legend li menu noframes noscript ol optgroup option pre q s samp
  select small span strike strong style sub sup table tbody td
  textarea tfoot th thead title tr tt u ul var'''.strip().split())

class HtmlElement(XmlElement):
    def __init__(self, *args, **kw):
        if 'cls' in kw:
            kw['class'] = kw.pop('cls')
        super(HtmlElement, self).__init__(*args, **kw)
        self._nonempty = (self.tag in _non_empty_html_tags)

class HtmlElementGen(XmlElementGen):
    elem_cls = HtmlElement
    def __init__(self, tag, argnames=(), **defaults):
        super(HtmlElementGen, self).__init__(tag)
        self.argnames = argnames
        self.defaults = defaults.items()

    def __call__(self, *args, **kw):
        if len(self.argnames) < len(args):
            raise ValueError(args)
        kw.update(zip(self.argnames, args))
        for key, value in self.defaults:
            kw.setdefault(key, value)
        return super(HtmlElementGen, self).__call__(**kw)


tag_definitions = '''
html head body title
script style
link: rel href type
meta: name content

h1 h2 h3 h4 h5 h6 h7 h8
ul ol li
br hr
em strong

pre code tt: class
p blockquote span sub sup big small center: class

table thead tbody tfoot: class
tr th td: class

a: href title
div: id class
form: action method
input: type name value
label: for
textarea: name
noscript

section nav
header footer
article aside hgroup
date: datetime
mark
'''
#@@ date accepting datetime instances + pubdate=bool

_g = globals()
for line in tag_definitions.strip().splitlines():
    if ':' in line:
        tags, argnames = line.split(':', 1)
    else:
        tags, argnames = line, ''
    argnames = argnames.strip().split()
    tags = tags.strip().split()

    for tag in tags:
        _g[tag.upper()] = HtmlElementGen(tag, argnames)

## A = HtmlElementGen('a', ['href', 'title']) ##
## def A(href=None, title=None, **kw):
##     #href = quote_plus(href)
##     if href:
##         href = href.replace(' ', '+')
##     return _A(href, title, **kw)
##

BOLD = HtmlElementGen('b')
ITALIC = HtmlElementGen('i')
STRIKE = HtmlElementGen('s')

entities = 'amp lt gt nbsp emsp ensp rarr larr ndash mdash bull tm copy trade'

for ent in entities.split():
    #_g[ent] =
    _g[ent.upper()] = XmlEntity(ent)


class DOCTYPE(object):
    @classmethod
    def __xmlnode__(cls):
        return cls.transitional

    html5 = raw_html('<!doctype html>\n')
    strict = raw_html('<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Strict//EN" '
                            '"http://www.w3.org/TR/xhtml1/DTD/xhtml1-strict.dtd">\n')
    html = raw_html('<!DOCTYPE HTML PUBLIC "-//W3C//DTD HTML 4.01//EN" '
                            '"http://www.w3.org/TR/html4/strict.dtd">\n')
    transitional = raw_html('<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Transitional//EN" '
                            '"http://www.w3.org/TR/xhtml1/DTD/xhtml1-transitional.dtd">\n')



LINK_css = HtmlElementGen('link', ['href'], rel="stylesheet", type='text/css')
LINK_rss = HtmlElementGen('link', ['href', 'title'], rel='alternate', type='application/rss+xml')
LINK_favicon = HtmlElementGen('link', ['href'], rel='shortcut icon')

def HTTP_EQUIV(name, content):
    return META(http_equiv=name, content=content)

META_noarchive = META("robots", "noarchive")

def META_html(encoding):
    return HTTP_EQUIV("Content-Type", "text/html; charset=%s" % encoding) #@@ canonicalize
META_html.__xmlnode__ = lambda: META_html('UTF-8')
#META_html = HTTP_EQUIV("Content-Type", "text/html; charset=UTF-8")

def SCRIPT_js(content):
    return HtmlElementGen('script', type="text/javascript")[
        #raw_html('<!--\n'),
        content,
        #raw_html('\n// -->')
    ]

def LINK_js(src):
    return HtmlElement('script', '', src=src, type="text/javascript")



def BUTTON(label, onclick):
    return HtmlElement('button', label, onclick=onclick)

#BUTTON = HtmlElementGen('input', ['name', 'value'], type='button')
IMG = HtmlElementGen('img', ['src', 'alt'], alt='')

INPUT_submit = HtmlElementGen('input', ['value'], type='submit')


def A_email(email, **kw):
    return A('mailto:%s' % email, **kw)[email]

def A_anchor(name):
    return A(name=name)['']

A_nofollow = HtmlElementGen('a', ['href', 'title'], rel='nofollow')

def PRE_flex(src, **kw):
    tag = PRE if '\n' in src else CODE
    return tag(**kw)[src]







_link_ga = SCRIPT_js(
    'var gaJsHost = (("https:" == document.location.protocol) ? "https://ssl." : "http://www.");' \
    '''document.write(unescape("%3Cscript src='" + gaJsHost + "google-analytics.com/ga.js' type='text/javascript'%3E%3C/script%3E"));'''
)

def GOOGLE_analytics(code, https=None):
    if https is None:
        link_ga = _link_ga
    else:
        assert isinstance(https, bool)
        src = ('https://ssl.' if https else 'http://www.') + 'google-analytics.com/ga.js'
        link_ga = LINK_js(src)
    return [link_ga, SCRIPT_js('try{_gat._getTracker("%s")._trackPageview();} catch(err) {}' % code)]



class HtmlDocument(XmlDocument):
    mime_type = 'text/html'
    def __init__(self, content, encoding='UTF-8'):
        XmlDocument.__init__(self, content, encoding)
        if not self.content.startswith('<!'):
            self.content = DOCTYPE.html5 + self.content

    def __unicode__(self):
        return self.content

    def encode(self, encoding):
        return self.content.encode(encoding)


class XHtmlDocument(XmlDocument):
    mime_type = 'application/xhtml+xml'
    def __init__(self, content, encoding='UTF-8', strict=False):
        doctype = DOCTYPE.strict if strict else DOCTYPE.transitional
        XmlDocument.__init__(self, [doctype, content], encoding)
















